# Mqtt.py
# -*- coding: utf-8 -*-
# cython: language_level=3
import json
import paho.mqtt.client as mqtt

class MQTTClient:
    def __init__(self, broker_address, port):
        """
        MQTT客户端初始化函数

        :param broker_address: MQTT代理服务器地址
        :param port: MQTT代理服务器端口
        """
        # 保存MQTT代理服务器地址和端口作为实例变量
        self.broker_address = broker_address
        self.port = port

        # 创建MQTT客户端实例，使用MQTTv311协议版本
        self.client = mqtt.Client(protocol=mqtt.MQTTv311)

        # 初始化接收到的消息变量
        self.received_message = None

        # 设置回调函数
        self.client.on_connect = self.on_connect  # 当客户端连接到代理时调用
        self.client.on_message = self.on_message  # 当客户端接收到消息时调用

    def on_connect(self, client, userdata, flags, rc):
        """
        当客户端连接到MQTT代理时调用的回调函数。

        参数:
        - client: 客户端实例，用于操作MQTT代理。
        - userdata: 在客户端初始化时定义的任意类型的数据。
        - flags: 连接应答的标志位。
        - rc: 连接结果的返回码，用于判断连接是否成功。

        此函数没有返回值。

        功能描述:
        - 如果连接成功（rc == 0），则打印成功连接的消息。
        - 如果连接失败，则打印失败的消息，并显示返回码。
        """
        if rc == 0:
            print("Connected to MQTT Broker!")
        else:
            print(f"Failed to connect, return code {rc}")

    def on_message(self, client, userdata, msg):
        """
        当接收到消息时调用的回调函数。

        参数:
        - client: MQTT客户端实例。
        - userdata: 用户定义的数据，未使用。
        - msg: 消息数据，包含主题（topic）和负载（payload）。

        此函数将解码消息负载，并将其存储为实例属性`received_message`，然后打印出接收到的消息和其对应的主题。
        """
        # 解码消息负载并存储为实例属性
        self.received_message = msg.payload.decode()
        # 打印接收到的消息和其对应的主题
        print(f"Received message: {self.received_message} on topic {msg.topic}")

    def connect(self):
        """
        Establishes a connection to the MQTT broker.

        This method calls the connect method of the self.client instance to establish a network connection to the MQTT broker,
        using the broker address and port number that were set during class initialization. After the connection is established,
        it starts the network loop using loop_start to keep the connection alive and handle MQTT protocol communications.
        """
        self.client.connect(self.broker_address, self.port)
        self.client.loop_start()

    def disconnect(self):
        """
        从服务器断开连接。

        该方法首先停止客户端的循环，然后断开与服务器的连接。
        """
        # 停止客户端的循环
        self.client.loop_stop()
        # 断开与服务器的连接
        self.client.disconnect()

    def publish(self, topic, message, qos=0):
        """
        发布消息到指定主题。

        参数:
        - topic: 消息发布的目标主题。
        - message: 要发布的消息内容，将被转换为JSON格式。
        - qos: 消息的服务质量等级，默认为0。

        此方法将消息内容序列化为JSON格式，并通过客户端发布到指定的主题上。
        """
        # 将消息内容转换为JSON格式并发布到指定主题
        self.client.publish(topic, json.dumps(message), qos=qos)

    def subscribe(self, topic, qos=0):
        """
        订阅指定的主题。

        参数:
        - topic (str): 要订阅的主题名称。
        - qos (int): 消息的质量服务等级，可选参数，默认为0。

        此方法调用client实例的subscribe方法来执行MQTT协议的订阅操作。
        """
        self.client.subscribe(topic, qos=qos)

    def wait_for_message(self, timeout=5):
        """
        等待接收消息，直到达到超时时间。

        本函数会持续检查是否接收到消息，直到消息到达或超出指定的超时时间。
        这种等待机制适用于需要同步处理消息的场景，通过轮询方式检查消息状态。

        参数:
        timeout (int): 超时时间（秒），默认为5秒。表示在没有接收到消息的情况下，函数将等待的最大时间。

        返回:
        接收到的消息内容，如果没有消息在指定时间内到达，则返回None。
        """

        # 导入time模块用于计算时间差和延时
        import time

        # 记录开始等待的时间
        start_time = time.time()

        # 当没有接收到消息且未超出超时时间时，持续等待
        while self.received_message is None and time.time() - start_time < timeout:
            # 短暂暂停，避免过高CPU占用
            time.sleep(0.1)

        # 返回接收到的消息，如果没有消息则返回None
        return self.received_message

